@@ -1,7 +1,4 @@
 """
-Assistants
----------
-
 Support for setting up and defining assistants that you can use in Kodexa
 """
-from .assistant import Assistant
+from .assistant import Assistant, AssistantContext
